@@ -47,8 +47,6 @@ gtk_html_control_data_new (GtkHTML *html, GtkWidget *vbox)
 	ncd->search_text             = NULL;
 	ncd->replace_text_search     = NULL;
 	ncd->replace_text_replace    = NULL;
-	ncd->has_spell_control_set   = FALSE;
-	ncd->language                = NULL;
 
 	spell_init (html, ncd);
 
@@ -78,8 +76,5 @@ gtk_html_control_data_destroy (GtkHTMLControlData *cd)
 	if (cd->gdk_painter) 
 		g_object_unref (cd->gdk_painter);
 
-	if (cd->languages)
-		CORBA_free (cd->languages);
-
 	g_free (cd);
 }
