@@ -21,6 +21,7 @@
 */
 
 #include <config.h> 
+#include <libgnome/gnome-i18n.h>
 #include <gnome.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include "search.h"
@@ -68,7 +69,7 @@ GtkHTMLSearchDialog *
 gtk_html_search_dialog_new (GtkHTML *html, GtkHTMLControlData *cd)
 {
 	GtkHTMLSearchDialog *dialog = g_new (GtkHTMLSearchDialog, 1);
-	GtkWidget *hbox;
+	GtkWidget *hbox, *vbox;
 
 	dialog->dialog         = GTK_DIALOG (gtk_dialog_new_with_buttons (_("Find"), NULL, 0,
 									  GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
@@ -88,11 +89,15 @@ gtk_html_search_dialog_new (GtkHTML *html, GtkHTMLControlData *cd)
 	gtk_box_pack_start (GTK_BOX (hbox), dialog->backward, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), dialog->case_sensitive, FALSE, FALSE, 0);
 
-	gtk_box_pack_start (GTK_BOX (dialog->dialog->vbox), dialog->entry, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (dialog->dialog->vbox), hbox, FALSE, FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (dialog->dialog->vbox), 6);
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_widget_show (vbox);
+	gtk_box_pack_start (GTK_BOX (vbox), dialog->entry, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog->dialog), 6);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog->dialog->vbox), 6);
+	gtk_box_set_spacing (GTK_BOX (dialog->dialog->vbox), 6);
+	gtk_box_pack_start (GTK_BOX (dialog->dialog->vbox), vbox, FALSE, FALSE, 0);
 	gtk_widget_show (dialog->entry);
 	gtk_widget_show_all (hbox);
 
