@@ -473,7 +473,7 @@ get_cursor_base (HTMLObject *self,
 static guint
 get_length (HTMLObject *self)
 {
-	return html_object_accepts_cursor (self) ? 1 : 0;
+	return 1;
 }
 
 static guint
@@ -498,7 +498,7 @@ select_range (HTMLObject *self,
 	gboolean selected;
 	gboolean changed;
 
-	selected = length > 0 || (length == -1 && start < html_object_get_length (self)) || html_object_is_container (self) ? TRUE : FALSE;
+	selected = length > 0 || (length == -1 && start < html_object_get_length (self)) ? TRUE : FALSE;
 	changed  = (! selected && self->selected) || (selected && ! self->selected) ? TRUE : FALSE;
 
 	self->selected = selected;
@@ -1413,7 +1413,7 @@ html_object_cursor_backward (HTMLObject *self, HTMLCursor *cursor)
 		return FALSE;
 
 	if (cursor->offset > 1 || (cursor->offset > 0 && (! (prev = html_object_prev_not_slave (self))
-							  || HTML_IS_CLUEALIGNED (prev) || !html_object_accepts_cursor (prev)))) {
+							  || HTML_IS_CLUEALIGNED (prev)))) {
 		cursor->offset --;
 		cursor->position --;
 		return TRUE;
@@ -1512,22 +1512,11 @@ next_object_uptree_cursor (HTMLObject *obj, HTMLObject * (*next_fn ) (HTMLObject
 /* go down in tree to leaf in way given by down_fn children */
 
 static HTMLObject *
-move_object_downtree_cursor (HTMLObject *obj, HTMLObject * (*down_fn ) (HTMLObject *), HTMLObject * (*next_fn ) (HTMLObject *))
+move_object_downtree_cursor (HTMLObject *obj, HTMLObject * (*down_fn ) (HTMLObject *))
 {
-	HTMLObject *last_obj = obj;
-
 	while ((obj = (*down_fn) (obj))) {
 		if (html_object_accepts_cursor (obj))
 			break;
-		last_obj = obj;
-	}
-
-	if (!obj && last_obj) {
-		obj = last_obj;
-
-		while ((obj = (*next_fn) (obj)))
-			if (html_object_accepts_cursor (obj))
-				break;
 	}
 
 	return obj;
@@ -1543,7 +1532,7 @@ move_object_cursor (HTMLObject *obj, gint *offset, gboolean forward,
 		gboolean found = FALSE;
 		if (((*offset == 0 && forward) || (*offset && !forward)) && html_object_is_container (obj))
 			if ((down = (*down_fn) (obj))) {
-				down = move_object_downtree_cursor (down, down_fn, next_fn);
+				down = move_object_downtree_cursor (down, down_fn);
 				if (down) {
 					if (html_object_is_container (down))
 						*offset = forward ? 0 : 1;
@@ -1563,7 +1552,7 @@ move_object_cursor (HTMLObject *obj, gint *offset, gboolean forward,
 					found = TRUE;
 				} else {
 					HTMLObject *down;
-					down = move_object_downtree_cursor (obj, down_fn, next_fn);
+					down = move_object_downtree_cursor (obj, down_fn);
 					if (down) {
 						if (html_object_is_container (down))
 							*offset = forward ? 0 : 1;
@@ -1648,7 +1637,7 @@ object_save_data (GQuark key_id, gpointer data, gpointer user_data)
 	/* printf ("object %s %s\n", g_quark_to_string (key_id), str); */
 	if (!str) {
 		/* printf ("save %s %s -> %s\n", state->save_data_class_name, g_quark_to_string (key_id), (gchar *) data); */
-		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">-->",
+		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">",
 						state->save_data_class_name, g_quark_to_string (key_id), data);
 		html_engine_set_class_data (state->engine, state->save_data_class_name, g_quark_to_string (key_id), data);
 	}
@@ -1664,12 +1653,12 @@ handle_object_data (gpointer key, gpointer value, gpointer data)
 	/* printf ("handle: %s %s %s %s\n", state->save_data_class_name, key, value, str); */
 	if (!str) {
 		/* printf ("clear\n"); */
-		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" clear=\"%s\">-->",
+		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" clear=\"%s\">",
 						state->save_data_class_name, key);
 		state->data_to_remove = g_slist_prepend (state->data_to_remove, key);
 	} else if (strcmp (value, str)) {
 		/* printf ("change\n"); */
-		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">-->",
+		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">",
 						state->save_data_class_name, key, str);
 		html_engine_set_class_data (state->engine, state->save_data_class_name, key, value);
 	}
