@@ -22,10 +22,11 @@
 #include <config.h>
 #include "gtkhtml-compat.h"
 
+#include <string.h>
 #include <gnome.h>
 #include <ctype.h>
 #include <libgnomeprint/gnome-print.h>
-#include <gal/unicode/gunicode.h>
+#include <libgnomeprint/gnome-print-paper.h>
 
 #include "htmlembedded.h"
 #include "gtkhtml-embedded.h"
@@ -41,75 +42,89 @@ static HTMLPainterClass *parent_class = NULL;
 /* The size of a pixel in the printed output, in points.  */
 #define PIXEL_SIZE .5
 
-/* The following macros are used to convert between the HTMLEngine coordinate
-   system (which uses integers) to the GnomePrint one (which uses doubles). */
-
-#define SCALE_ENGINE_TO_GNOME_PRINT(x) ((printer->scale * (x)) / 1024.0)
-#define SCALE_GNOME_PRINT_TO_ENGINE(x) ((gint) (((x) * 1024.0 / printer->scale) + 0.5))
-
-/* Hm, this might need fixing.  */
-#define SPACING_FACTOR 1.2
-
-
-/* This is just a quick hack until GnomePrintContext is fixed to hold the paper
-   size.  */
-
-static const GnomePaper *paper = NULL;
-
 static void
-insure_paper (HTMLPrinter *printer)
+insure_config (HTMLPrinter *p)
 {
-	if (paper != NULL)
-		return;
-
-	if (printer->print_master) {
-		paper = gnome_print_master_get_paper (printer->print_master);
-	}
-	if (!paper)
-		paper = gnome_paper_with_name (_("US-Letter"));
-	g_assert (paper != NULL);
+	if (!p->config)
+		p->config = p->master ? gnome_print_job_get_config (p->master) : gnome_print_config_default ();
 }
 
 static gdouble
 printer_get_page_height (HTMLPrinter *printer)
 {
-	insure_paper (printer);
-	return gnome_paper_psheight (paper);
+	gdouble width, height = 0.0;
+
+	insure_config (printer);
+	gnome_print_config_get_page_size (printer->config, &width, &height);
+
+	return height;
 }
 
 static gdouble
 printer_get_page_width (HTMLPrinter *printer)
 {
-	insure_paper (printer);
-	return gnome_paper_pswidth (paper);
+	gdouble width = 0.0, height;
+
+	insure_config (printer);
+	gnome_print_config_get_page_size (printer->config, &width, &height);
+
+	return width;
 }
 
+/* .5" in PS units */
+#define TEMP_MARGIN 72*.5
+
 static gdouble
 get_lmargin (HTMLPrinter *printer)
 {
-	insure_paper (printer);
-	return gnome_paper_lmargin (paper) / 2;
+	/*gdouble lmargin = 0.0;
+
+	insure_config (printer);
+	gnome_print_config_get_double (printer->config, GNOME_PRINT_KEY_PAGE_MARGIN_LEFT, &lmargin);
+
+	printf ("lmargin %f\n", lmargin);
+	return lmargin;*/
+
+	return TEMP_MARGIN;
 }
 
 static gdouble
 get_rmargin (HTMLPrinter *printer)
 {
-	insure_paper (printer);
-	return gnome_paper_rmargin (paper) / 2;
+	/*gdouble rmargin = 0.0;
+
+	insure_config (printer);
+	gnome_print_config_get_double (printer->config, GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT, &rmargin);
+
+	return rmargin;*/
+
+	return TEMP_MARGIN;
 }
 
 static gdouble
 get_tmargin (HTMLPrinter *printer)
 {
-	insure_paper (printer);
-	return gnome_paper_tmargin (paper) / 2;
+	/* gdouble tmargin = 0.0;
+
+	insure_config (printer);
+	gnome_print_config_get_double (printer->config, GNOME_PRINT_KEY_PAGE_MARGIN_TOP, &tmargin);
+
+	return tmargin; */
+
+	return TEMP_MARGIN;
 }
 
 static gdouble
 get_bmargin (HTMLPrinter *printer)
 {
-	insure_paper (printer);
-	return gnome_paper_bmargin (paper) / 2;
+	/* gdouble bmargin = 0.0;
+
+	insure_config (printer);
+	gnome_print_config_get_double (printer->config, GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM, &bmargin);
+
+	return bmargin; */
+
+	return TEMP_MARGIN;
 }
 
 gdouble
@@ -153,18 +168,23 @@ gnome_print_coordinates_to_engine (HTMLPrinter *printer,
 /* GtkObject methods.  */
 
 static void
-finalize (GtkObject *object)
+finalize (GObject *object)
 {
 	HTMLPrinter *printer;
 
 	printer = HTML_PRINTER (object);
 
-	if (printer->print_context != NULL) {
-		gnome_print_context_close (printer->print_context);
-		gtk_object_unref (GTK_OBJECT (printer->print_context));
+	if (printer->config != NULL) {
+		/* FIXME g_object_unref (printer->config); */
+		printer->config = NULL;
+	}
+	if (printer->context != NULL) {
+		gnome_print_context_close (printer->context);
+		g_object_unref (printer->context);
+		printer->context = NULL;
 	}
 
-	(* GTK_OBJECT_CLASS (parent_class)->finalize) (object);
+	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
 
@@ -182,7 +202,7 @@ begin (HTMLPainter *painter,
 #endif
 	printer = HTML_PRINTER (painter);
 	g_return_if_fail (printer);
-	pc      = printer->print_context;
+	pc      = printer->context;
 	g_return_if_fail (pc);
 
 	gnome_print_beginpage (pc, "page");
@@ -220,10 +240,10 @@ end (HTMLPainter *painter)
 	HTMLPrinter *printer;
 
 	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->print_context != NULL);
+	g_return_if_fail (printer->context != NULL);
 
-	gnome_print_grestore (printer->print_context);
-	gnome_print_showpage (printer->print_context);
+	gnome_print_grestore (printer->context);
+	gnome_print_showpage (printer->context);
 }
 
 static void
@@ -250,9 +270,9 @@ set_pen (HTMLPainter *painter,
 	HTMLPrinter *printer;
 
 	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->print_context != NULL);
+	g_return_if_fail (printer->context != NULL);
 
-	gnome_print_setrgbcolor (printer->print_context,
+	gnome_print_setrgbcolor (printer->context,
 				 color->red / 65535.0, color->green / 65535.0, color->blue / 65535.0);
 }
 
@@ -268,7 +288,7 @@ static void
 prepare_rectangle (HTMLPainter *painter, gint _x, gint _y, gint w, gint h)
 {
 	HTMLPrinter *printer = HTML_PRINTER (painter);
-	GnomePrintContext *context = printer->print_context;
+	GnomePrintContext *context = printer->context;
 	gdouble x;
 	gdouble y;
 	gdouble width;
@@ -292,7 +312,7 @@ static void
 do_rectangle (HTMLPainter *painter, gint x, gint y, gint w, gint h, gint lw)
 {
 	HTMLPrinter *printer = HTML_PRINTER (painter);
-	GnomePrintContext *context = printer->print_context;
+	GnomePrintContext *context = printer->context;
 
 	gnome_print_setlinewidth (context, SCALE_ENGINE_TO_GNOME_PRINT (lw) * PIXEL_SIZE);
 	prepare_rectangle (painter, x, y, w, h);
@@ -305,7 +325,7 @@ set_clip_rectangle (HTMLPainter *painter,
 		    gint width, gint height)
 {
 	prepare_rectangle (painter, x, y, width, height);
-	gnome_print_clip (HTML_PRINTER (painter)->print_context);
+	gnome_print_clip (HTML_PRINTER (painter)->context);
 }
 
 /* HTMLPainter drawing functions.  */
@@ -320,18 +340,18 @@ draw_line (HTMLPainter *painter,
 	double printer_x2, printer_y2;
 
 	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->print_context != NULL);
+	g_return_if_fail (printer->context != NULL);
 
 	html_printer_coordinates_to_gnome_print (printer, x1, y1, &printer_x1, &printer_y1);
 	html_printer_coordinates_to_gnome_print (printer, x2, y2, &printer_x2, &printer_y2);
 
-	gnome_print_setlinewidth (printer->print_context, PIXEL_SIZE);
+	gnome_print_setlinewidth (printer->context, PIXEL_SIZE);
 
-	gnome_print_newpath (printer->print_context);
-	gnome_print_moveto (printer->print_context, printer_x1, printer_y1);
-	gnome_print_lineto (printer->print_context, printer_x2, printer_y2);
+	gnome_print_newpath (printer->context);
+	gnome_print_moveto (printer->context, printer_x1, printer_y1);
+	gnome_print_lineto (printer->context, printer_x2, printer_y2);
 
-	gnome_print_stroke (printer->print_context);
+	gnome_print_stroke (printer->context);
 }
 
 static void
@@ -351,7 +371,7 @@ draw_panel (HTMLPainter *painter,
 	    gint bordersize)
 {
 	HTMLPrinter *printer = HTML_PRINTER (painter);
-	GnomePrintContext *pc = printer->print_context;
+	GnomePrintContext *pc = printer->context;
 	GdkColor *col1 = NULL, *col2 = NULL;
 	GdkColor dark, light;
 	gdouble x;
@@ -434,8 +454,8 @@ draw_background (HTMLPainter *painter,
 
 	printer = HTML_PRINTER (painter);
 	g_return_if_fail (printer);
-	pc = printer->print_context;
-	g_return_if_fail (printer->print_context);
+	pc = printer->context;
+	g_return_if_fail (printer->context);
 
 	width = SCALE_ENGINE_TO_GNOME_PRINT  (pix_width);
 	height = SCALE_ENGINE_TO_GNOME_PRINT (pix_height);
@@ -457,11 +477,28 @@ draw_background (HTMLPainter *painter,
 }
 
 static void
-draw_pixmap (HTMLPainter *painter,
-	     GdkPixbuf *pixbuf,
-	     gint x, gint y,
-	     gint scale_width, gint scale_height,
-	     const GdkColor *color)
+print_pixbuf (GnomePrintContext *pc, GdkPixbuf *pixbuf)
+{
+	if (!pixbuf || (gdk_pixbuf_get_colorspace (pixbuf) != GDK_COLORSPACE_RGB))
+		return;
+	
+	if (gdk_pixbuf_get_has_alpha (pixbuf)) {
+		gnome_print_rgbaimage (pc, 
+				       gdk_pixbuf_get_pixels (pixbuf),
+				       gdk_pixbuf_get_width (pixbuf),
+				       gdk_pixbuf_get_height (pixbuf),
+				       gdk_pixbuf_get_rowstride (pixbuf));
+	} else {
+		gnome_print_rgbimage (pc, 
+				      gdk_pixbuf_get_pixels (pixbuf),
+				      gdk_pixbuf_get_width (pixbuf),
+				      gdk_pixbuf_get_height (pixbuf),
+				      gdk_pixbuf_get_rowstride (pixbuf));
+	}
+}
+
+static void
+draw_pixmap (HTMLPainter *painter, GdkPixbuf *pixbuf, gint x, gint y, gint scale_width, gint scale_height, const GdkColor *color)
 {
 	HTMLPrinter *printer;
 	gint width, height;
@@ -469,7 +506,7 @@ draw_pixmap (HTMLPainter *painter,
 	double print_scale_width, print_scale_height;
 
 	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->print_context != NULL);
+	g_return_if_fail (printer->context != NULL);
 
 	width = gdk_pixbuf_get_width (pixbuf);
 	height = gdk_pixbuf_get_height (pixbuf);
@@ -479,108 +516,98 @@ draw_pixmap (HTMLPainter *painter,
 	print_scale_width  = SCALE_ENGINE_TO_GNOME_PRINT (scale_width);
 	print_scale_height = SCALE_ENGINE_TO_GNOME_PRINT (scale_height);
 
-	gnome_print_gsave (printer->print_context);
-	gnome_print_translate (printer->print_context, print_x, print_y - print_scale_height);
-	gnome_print_scale (printer->print_context, print_scale_width, print_scale_height);
-	gnome_print_pixbuf (printer->print_context, pixbuf);
-	gnome_print_grestore (printer->print_context);
+	gnome_print_gsave (printer->context);
+	gnome_print_translate (printer->context, print_x, print_y - print_scale_height);
+	gnome_print_scale (printer->context, print_scale_width, print_scale_height);
+	print_pixbuf (printer->context, pixbuf);
+	gnome_print_grestore (printer->context);
 }
 
 static void
-fill_rect (HTMLPainter *painter,
-	   gint x, gint y,
-	   gint width, gint height)
+fill_rect (HTMLPainter *painter, gint x, gint y, gint width, gint height)
 {
 	HTMLPrinter *printer;
 	double printer_x, printer_y;
 	double printer_width, printer_height;
 
 	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->print_context != NULL);
+	g_return_if_fail (printer->context != NULL);
 
 	printer_width = SCALE_ENGINE_TO_GNOME_PRINT (width);
 	printer_height = SCALE_ENGINE_TO_GNOME_PRINT (height);
 
 	html_printer_coordinates_to_gnome_print (printer, x, y, &printer_x, &printer_y);
 
-	gnome_print_newpath (printer->print_context);
-	gnome_print_moveto (printer->print_context, printer_x, printer_y);
-	gnome_print_lineto (printer->print_context, printer_x + printer_width, printer_y);
-	gnome_print_lineto (printer->print_context, printer_x + printer_width, printer_y - printer_height);
-	gnome_print_lineto (printer->print_context, printer_x, printer_y - printer_height);
-	gnome_print_lineto (printer->print_context, printer_x, printer_y);
-	gnome_print_closepath (printer->print_context);
+	gnome_print_newpath (printer->context);
+	gnome_print_moveto (printer->context, printer_x, printer_y);
+	gnome_print_lineto (printer->context, printer_x + printer_width, printer_y);
+	gnome_print_lineto (printer->context, printer_x + printer_width, printer_y - printer_height);
+	gnome_print_lineto (printer->context, printer_x, printer_y - printer_height);
+	gnome_print_lineto (printer->context, printer_x, printer_y);
+	gnome_print_closepath (printer->context);
 
-	gnome_print_fill (printer->print_context);
+	gnome_print_fill (printer->context);
 }
 
-static void
-draw_text (HTMLPainter *painter,
-	   gint x, gint y,
-	   const gchar *text,
-	   gint len)
+static gint
+draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, GList *items, GList *glyphs, gint start_byte_offset)
 {
 	GnomeFont *font;
 	HTMLPrinter *printer;
+	gint bytes;
 	gdouble print_x, print_y;
-	gchar *end, *text_tmp;
+	double text_width, asc, dsc;
 
 	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->print_context != NULL);
+	g_return_val_if_fail (printer->context != NULL, 0);
 
+
+	bytes = g_utf8_offset_to_pointer (text, len) - text;
+	font = html_painter_get_font (painter, painter->font_face, painter->font_style);
+	dsc = -gnome_font_get_descender (font);
+	asc = gnome_font_get_ascender (font);
 	html_printer_coordinates_to_gnome_print (printer, x, y, &print_x, &print_y);
 
-	gnome_print_newpath (printer->print_context);
-	gnome_print_moveto (printer->print_context, print_x, print_y);
-
-	/* Oh boy, this sucks so much.  The GnomePrint API could be improved to
-           avoid this.  */
-	end = g_utf8_offset_to_pointer (text, len);
-	text_tmp = alloca (end - text + 1);
-	memcpy (text_tmp, text, end - text);
-	text_tmp [end - text] = '\0';
-
-	font = html_painter_get_font (painter, painter->font_face, painter->font_style);
-	gnome_print_setfont (printer->print_context, font);
-	gnome_print_show (printer->print_context, text_tmp);
+	gnome_print_newpath (printer->context);
+	gnome_print_moveto (printer->context, print_x, print_y);
+	gnome_print_setfont (printer->context, font);
+	gnome_print_show_sized (printer->context, text, bytes);
 
+	text_width = gnome_font_get_width_utf8_sized (font, text, bytes);
 	if (painter->font_style & (GTK_HTML_FONT_STYLE_UNDERLINE | GTK_HTML_FONT_STYLE_STRIKEOUT)) {
-		double text_width;
-		double ascender, descender;
 		double y;
 
-		gnome_print_gsave (printer->print_context);
+		gnome_print_gsave (printer->context);
 
 		/* FIXME: We need something in GnomeFont to do this right.  */
-		gnome_print_setlinewidth (printer->print_context, 1.0);
-		gnome_print_setlinecap (printer->print_context, GDK_CAP_BUTT);
+		gnome_print_setlinewidth (printer->context, 1.0);
+		gnome_print_setlinecap (printer->context, GDK_CAP_BUTT);
 
-		text_width = gnome_font_get_width_utf8_sized (font, text, len);
 		if (painter->font_style & GTK_HTML_FONT_STYLE_UNDERLINE) {
-			descender = gnome_font_get_descender (font);
 			y = print_y + gnome_font_get_underline_position (font);
 
-			gnome_print_newpath (printer->print_context);
-			gnome_print_moveto (printer->print_context, print_x, y);
-			gnome_print_lineto (printer->print_context, print_x + text_width, y);
-			gnome_print_setlinewidth (printer->print_context,
+			gnome_print_newpath (printer->context);
+			gnome_print_moveto (printer->context, print_x, y);
+			gnome_print_lineto (printer->context, print_x + text_width, y);
+			gnome_print_setlinewidth (printer->context,
 						  gnome_font_get_underline_thickness (font));
-			gnome_print_stroke (printer->print_context);
+			gnome_print_stroke (printer->context);
 		}
 
 		if (painter->font_style & GTK_HTML_FONT_STYLE_STRIKEOUT) {
-			ascender = gnome_font_get_ascender (font);
-			y = print_y + ascender / 2.0;
-			gnome_print_newpath (printer->print_context);
-			gnome_print_moveto (printer->print_context, print_x, y);
-			gnome_print_lineto (printer->print_context, print_x + text_width, y);
-			gnome_print_setlinewidth (printer->print_context,
+			y = print_y + asc / 2.0;
+			gnome_print_newpath (printer->context);
+			gnome_print_moveto (printer->context, print_x, y);
+			gnome_print_lineto (printer->context, print_x + text_width, y);
+			gnome_print_setlinewidth (printer->context,
 						  gnome_font_get_underline_thickness (font));
-			gnome_print_stroke (printer->print_context);
+			gnome_print_stroke (printer->context);
 		}
 
-		gnome_print_grestore (printer->print_context);
+		gnome_print_grestore (printer->context);
 	}
+
+	return SCALE_GNOME_PRINT_TO_ENGINE (text_width);
 }
 
 static void
@@ -591,19 +618,19 @@ draw_embedded (HTMLPainter *p, HTMLEmbedded *o, gint x, gint y)
 	GtkWidget *embedded_widget;
 
 	html_printer_coordinates_to_gnome_print (printer, x, y, &print_x, &print_y);
-	gnome_print_gsave(printer->print_context); 
+	gnome_print_gsave(printer->context); 
 
-	gnome_print_translate(printer->print_context, 
+	gnome_print_translate(printer->context, 
 			      print_x, print_y - o->height * PIXEL_SIZE);
  
 	embedded_widget = html_embedded_get_widget(o);
 	if (embedded_widget && GTK_IS_HTML_EMBEDDED (embedded_widget)) {
-		gtk_signal_emit_by_name(GTK_OBJECT (embedded_widget), 
-					"draw_print", 
-					printer->print_context);
+		g_signal_emit_by_name(GTK_OBJECT (embedded_widget), 0,
+				      "draw_print", 
+				      printer->context);
 	}
 
-	gnome_print_grestore(printer->print_context); 
+	gnome_print_grestore(printer->context); 
 }
 
 static void
@@ -614,69 +641,43 @@ draw_shade_line (HTMLPainter *painter,
 	HTMLPrinter *printer;
 
 	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->print_context != NULL);
+	g_return_if_fail (printer->context != NULL);
 
 	/* FIXME */
 }
 
-static guint
-calc_ascent (HTMLPainter *painter,
-	     GtkHTMLFontStyle style,
-	     HTMLFontFace *face)
+static void
+calc_text_size (HTMLPainter *painter, const gchar *text, guint len, GList *items, GList *glyphs, gint start_byte_offset,
+		GtkHTMLFontStyle style, HTMLFontFace *face, gint *width, gint *asc, gint *dsc)
 {
 	HTMLPrinter *printer;
 	GnomeFont *font;
-	double ascender;
 
 	printer = HTML_PRINTER (painter);
-	g_return_val_if_fail (printer->print_context != NULL, 0);
+	g_return_if_fail (printer->context != NULL);
 
 	font = html_painter_get_font (painter, face, style);
-	g_return_val_if_fail (font != NULL, 0);
+	g_return_if_fail (font != NULL);
 
-	ascender = gnome_font_get_ascender (font) * SPACING_FACTOR;
-	return SCALE_GNOME_PRINT_TO_ENGINE (ascender);
+	*width = SCALE_GNOME_PRINT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, text,
+									       g_utf8_offset_to_pointer (text, len) - text));
+	*asc = SCALE_GNOME_PRINT_TO_ENGINE (gnome_font_get_ascender (font));
+	*dsc = SCALE_GNOME_PRINT_TO_ENGINE (-gnome_font_get_descender (font));
 }
 
-static guint
-calc_descent (HTMLPainter *painter,
-	      GtkHTMLFontStyle style,
-	      HTMLFontFace *face)
+static void
+calc_text_size_bytes (HTMLPainter *painter, const gchar *text, guint len, GList *items, GList *glyphs, gint start_byte_offset,
+		      HTMLFont *font, GtkHTMLFontStyle style, gint *width, gint *asc, gint *dsc)
 {
 	HTMLPrinter *printer;
-	GnomeFont *font;
-	double descender;
 
 	printer = HTML_PRINTER (painter);
-	g_return_val_if_fail (printer->print_context != NULL, 0);
+	g_return_if_fail (printer->context != NULL);
+	g_return_if_fail (font != NULL);
 
-	font = html_painter_get_font (painter, face, style);
-	g_return_val_if_fail (font != NULL, 0);
-
-	descender = gnome_font_get_descender (font) * SPACING_FACTOR;
-	return SCALE_GNOME_PRINT_TO_ENGINE (descender);
-}
-
-static guint
-calc_text_width (HTMLPainter *painter,
-		 const gchar *text,
-		 guint len,
-		 GtkHTMLFontStyle style,
-		 HTMLFontFace *face)
-{
-	HTMLPrinter *printer;
-	GnomeFont *font;
-	double width;
-
-	printer = HTML_PRINTER (painter);
-	g_return_val_if_fail (printer->print_context != NULL, 0);
-
-	font = html_painter_get_font (painter, face, style);
-	g_return_val_if_fail (font != NULL, 0);
-
-	width = gnome_font_get_width_utf8_sized (font, text, len);
-
-	return SCALE_GNOME_PRINT_TO_ENGINE (width);
+	*width = SCALE_GNOME_PRINT_TO_ENGINE (gnome_font_get_width_utf8_sized (font->data, text, len));
+	*asc = SCALE_GNOME_PRINT_TO_ENGINE (gnome_font_get_ascender (font->data));
+	*dsc = SCALE_GNOME_PRINT_TO_ENGINE (-gnome_font_get_descender (font->data));
 }
 
 static guint
@@ -696,58 +697,47 @@ get_font_size (HTMLPrinter *printer, gboolean points, gdouble size)
 static HTMLFont *
 alloc_font (HTMLPainter *painter, gchar *face, gdouble size, gboolean points, GtkHTMLFontStyle style)
 {
-	HTMLPrinter *printer = HTML_PRINTER  (painter);
+	HTMLPrinter *printer = HTML_PRINTER (painter);
 	GnomeFontWeight weight;
 	GnomeFont *font;
 	gboolean italic;
-	gchar *family = NULL;
 
 	weight = (style & GTK_HTML_FONT_STYLE_BOLD) ? GNOME_FONT_BOLD : GNOME_FONT_BOOK;
 	italic = (style & GTK_HTML_FONT_STYLE_ITALIC);
 
-	/* gnome-print is case sensitive - need to be fixed */
-	if (face && *face) {
-		gchar *s;
-
-		s = family = html_font_manager_get_attr (face, 2);
-
-		/* capitalize */
-		*s = toupper (*s);
-		s++;
-		while (*s) {
-			*s = tolower (*s);
-			s++;
-		}
-	}
-
-	font = gnome_font_new_closest (family ? family : (style & GTK_HTML_FONT_STYLE_FIXED ? "Courier" : "Helvetica"),
-				       weight, italic, get_font_size (printer, points, size));
-	g_free (family);
-
+	font = gnome_font_find_closest_from_weight_slant (face ? face : (style & GTK_HTML_FONT_STYLE_FIXED ? "Monospace" : "Sans"),
+							  weight, italic, get_font_size (printer, points, size));
 	if (font == NULL) {
 		GList *family_list;
 
 		family_list = gnome_font_family_list ();
 		if (family_list && family_list->data) {
-			font = gnome_font_new_closest (family_list->data,
-						       weight, italic, get_font_size (printer, points, size));
+			font = gnome_font_find_closest_from_weight_slant (family_list->data, weight, italic, get_font_size (printer, points, size));
 			gnome_font_family_list_free (family_list);
 		}
 	}
 
-	return font ? html_font_new (font, gnome_font_get_width_utf8_sized (font, " ", 1)) : NULL;
+	return font ? html_font_new (font,
+				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, " ", 1)/HTML_PRINTER (printer)->scale),
+				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, "\xc2\xa0", 2)/HTML_PRINTER (printer)->scale),
+				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, "\t", 1)/HTML_PRINTER (printer)->scale),
+				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, HTML_BLOCK_CITE, strlen (HTML_BLOCK_CITE))
+								       /HTML_PRINTER (printer)->scale),
+				     SCALE_GNOME_PRINT_FONT_TO_ENGINE (gnome_font_get_width_utf8_sized (font, HTML_BLOCK_INDENT, strlen (HTML_BLOCK_INDENT))
+								       /HTML_PRINTER (printer)->scale))
+		: NULL;
 }
 
 static void
 ref_font (HTMLPainter *painter, HTMLFont *font)
 {
-	gtk_object_ref (GTK_OBJECT (font->data));
+	g_object_ref (font->data);
 }
 
 static void
 unref_font (HTMLPainter *painter, HTMLFont *font)
 {
-	gtk_object_unref (GTK_OBJECT (font->data));
+	g_object_unref (font->data);
 }
 
 static guint
@@ -761,24 +751,27 @@ get_page_height (HTMLPainter *painter, HTMLEngine *e)
 {
 	return html_printer_get_page_height (HTML_PRINTER (painter));
 }
-
+
 static void
-init (GtkObject *object)
+html_printer_init (GObject *object)
 {
 	HTMLPrinter *printer;
 
 	printer                = HTML_PRINTER (object);
-	printer->print_context = NULL;
+	printer->context       = NULL;
+	printer->config        = NULL;
 	printer->scale         = 1.0;
 }
 
 static void
-class_init (GtkObjectClass *object_class)
+html_printer_class_init (GObjectClass *object_class)
 {
 	HTMLPainterClass *painter_class;
 
 	painter_class = HTML_PAINTER_CLASS (object_class);
 
+	parent_class = g_type_class_ref (HTML_TYPE_PAINTER);
+
 	object_class->finalize = finalize;
 
 	painter_class->begin = begin;
@@ -788,9 +781,8 @@ class_init (GtkObjectClass *object_class)
 	painter_class->unref_font = unref_font;
 	painter_class->alloc_color = alloc_color;
 	painter_class->free_color = free_color;
-	painter_class->calc_ascent = calc_ascent;
-	painter_class->calc_descent = calc_descent;
-	painter_class->calc_text_width = calc_text_width;
+	painter_class->calc_text_size = calc_text_size;
+	painter_class->calc_text_size_bytes = calc_text_size_bytes;
 	painter_class->set_pen = set_pen;
 	painter_class->get_black = get_black;
 	painter_class->draw_line = draw_line;
@@ -807,44 +799,41 @@ class_init (GtkObjectClass *object_class)
 	painter_class->draw_embedded = draw_embedded;
 	painter_class->get_page_width = get_page_width;
 	painter_class->get_page_height = get_page_height;
-
-	parent_class = gtk_type_class (html_painter_get_type ());
 }
 
-GtkType
+GType
 html_printer_get_type (void)
 {
-	static GtkType type = 0;
+	static GType html_printer_type = 0;
 
-	if (type == 0) {
-		static const GtkTypeInfo info = {
-			"HTMLPrinter",
-			sizeof (HTMLPrinter),
+	if (html_printer_type == 0) {
+		static const GTypeInfo html_printer_info = {
 			sizeof (HTMLPrinterClass),
-			(GtkClassInitFunc) class_init,
-			(GtkObjectInitFunc) init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
+			NULL,
+			NULL,
+			(GClassInitFunc) html_printer_class_init,
+			NULL,
+			NULL,
+			sizeof (HTMLPrinter),
+			1,
+			(GInstanceInitFunc) html_printer_init,
 		};
-
-		type = gtk_type_unique (HTML_TYPE_PAINTER, &info);
+		html_printer_type = g_type_register_static (HTML_TYPE_PAINTER, "HTMLPrinter", &html_printer_info, 0);
 	}
 
-	return type;
+	return html_printer_type;
 }
 
-
 HTMLPainter *
-html_printer_new (GnomePrintContext *print_context, GnomePrintMaster *print_master)
+html_printer_new (GnomePrintContext *context, GnomePrintJob *master)
 {
 	HTMLPrinter *new;
 
-	new = gtk_type_new (html_printer_get_type ());
+	new = g_object_new (HTML_TYPE_PRINTER, NULL);
 
-	gtk_object_ref (GTK_OBJECT (print_context));
-	new->print_context = print_context;
-	new->print_master = print_master;
+	g_object_ref (context);
+	new->context = context;
+	new->master = master;
 
 	return HTML_PAINTER (new);
 }
